#!/usr/local/bin/perl

$_ = '$Revision: 1.16 $';
($version) = m#:\s*([\d.]*)#;

#################################################################
# WebbedGed 
# Translates gedcom files to html based on the gedcom 5.5 standard.
# This work was originated from Dan Pidcock's gedcomToHtml, v1.45.

# Copyright (C) 1999 Paul Rawlins

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# The license may also be obtained online at 
# http://www.fsf.org/copyleft/gpl.html.



#################################################################
# Configuration options.  If you need to customize any of these options,
# the preferred method is to copy the appropriate lines to the file 
# WebbedGed.cfg located in the current directory and make the modifications
# there.  This maintains your customizations when new versions of 
# the code are released.

$family_table = 1;        # print family tree type table
$print_family = 0;        # print detailed family info
$print_notes = 1;         # Print an individual's notes
$hot_links = 1;           # Convert URL's in notes to hot links
$check_images = 1;        # Check for images in $out_dir/$photo_dir
$private = 1;             # Make birth information private
$lds_ord = 1;             # Include LDS ordinances
$ped_generations = 5;     # Number of generations for pedigree chart
$debug = 0;               # Enable debug statements to print
$out_dir = "html";        # Directory where the HTML files will be stored
$photo_dir = "pics";      # Directory where the individual's picture files are
$root_id = 1;             # RIN ID of individual to be at root of ped chart
$add_gedcom_link = 1;     # Flag indicating whether to add link to gedcom data
$gedcom_link_path = ".."; # Path to gedcom data (no trailing slash)
$gedcom_link_ext = ".gz"; # Additional extention added to gedcom data file
$create_gendex = 0;       # Create GENDEX output file

# The file names for the frame-related files
$top_frame_name = "WebbedGed_frame.html";
$top_form_name  = "form.html";

# the tree gifs.  Note lack of trailing /.
$treepic_path = "../gif";
$gif_path = "../gif";

# The strings that are printed into files - change for different
# languages

$str_birth="Birth:";
$str_death="Death:";
$str_burial="Burial:";
$str_occupation="Occupation:";
$str_private="(Private)";
$str_father="Father:";
$str_mother="Mother:";
$str_child="Child";
$str_notes="Notes:";
$str_bap="Baptism:";
$str_end="Endowment:";
$str_slgc="Seal Parents:";
$str_slgs="Sealed";
$str_all_surnames="All surnames in the tree";
$str_people="People";
$str_surnames="Surnames";
$str_married="Married";
$str_on="on ";
$str_at="at ";
$str_list_of="List of";
$str_lpeople="List of people";
$str_lsurnames="List of surnames";
$str_people_and="people and";
$str_unique_names="unique names";
$str_m="m.";

#######################################################################
# Leave the rest alone unless you know what's happenin'

print "WebbedGed $version (c) Paul Rawlins 1999\n";

#if ($#ARGV != 0) {
#    die "usage: gedcomToHTML.pl <gedcom file>\n";
#}

$gedfile = $ARGV[0];

print "Gedcom file $gedfile\n";

if (-e "WebbedGed.cfg") {
  if (open (CFG, "<WebbedGed.cfg")) {
    while (<CFG>) {
      chomp;
      if (/^\s*\$([a-zA-Z0-9_]+)\s*=\s*\"*([^\";]+)\"*\s*;/) {
        ${$1} = $2;
        print "Using: $1 = $2\n";
      }
    }
  }
  else {
   warn "Cannot open config file.  Using defaults.";
  } 
}

if ($debug) { open (DBUG_OUT, ">dbug.out");}

$in = "nothing";
$in1 = "nothing";

# Get this year in 4 figure format
$curr_year = 1900 + (localtime(time))[5];

#######################################################################
# Make the directory and chdir to it

if (!-e $out_dir) {
    unless (mkdir($out_dir,0755)) {
        die "Couldn't create $out_dir directory\n";
    }
}

#################################################################
# Read the gedcom file in and created the individual and family 
# data structures.
print "Reading information\n";
$| = 1; # flushing for progress report.
$num_indivs = 0;
$num_families = 0;
$no_br_cont = 0;
while (<>) {
  # Remove newline
  chomp;
  
  if (/^\s*0.*/) {              # a 0 line
    if ($in1 != 0)              # reset in1
      {$in1 = "nothing";}
    if (/^\s*0\s*HEAD/) {       # header
      $in = "header";
      #print "Header $_";
    }
    elsif (/^\s*0\s@(.*)@\sFAM/) { # family record
      $in = "family_record";
      $fam_id = $1;
      $fams{$fam_id}++;
      $famc_cnt = 1;
      $num_families++;
      &show_reading_status;
      #print "Family $_";
    }
    elsif (/^\s*0\s@(.*)@\sINDI/) { # individual record
      $in = "individual_record";
      ($indiv_id = $1) =~ tr/I/i/;
      $indivs{$indiv_id}++;
      $num_indivs++;
      $num_fams = 0;            # number of spouses
      &show_reading_status;
      #print "Individual $_";
    }
    elsif (/^\s*0\sTRLR/) {     # end of file
      $in = "nothing";
    }
    elsif (/^\s*0\s_EVENT_DEFN/) { # custom event
      $in = "nothing";
    }
    elsif (/^\s*0\s@(.*)@\sSUBM/) { # submission record - just ignore
      $in = "nothing";
    }
    elsif (/^\s*0\s@(.*)@\sSOUR/) { # source record - just ignore
      $in = "nothing";
    }
    elsif (/^\s*0\s@(.*)@\sNOTE/) { # note record - linked to an indi
      $note_id = $1;
      # Separate notes in separate lines with line break.
      $note{$note_id} = $note{$note_id}."<br>";
      $in = "note_record";
    }
    else {
      $in = "nothing";
      print "Don't understand this 0 line: $_";
    }
  }
  elsif (/^\s*1\s(.*)/) {       # a 1 line
    $rol = $1;
    $in1 = "nothing";
    if ($in eq "header") {
      ;                         # ignore the header
    }
    elsif ($in eq "family_record") { 
      #print "\tfamily $fam_id $rol\n";
      if ($rol =~ /HUSB\s@(.*)@/) {
        ($fam_husb{$fam_id} = $1) =~ tr/I/i/;
      }
      if ($rol =~ /WIFE\s@(.*)@/) {
        ($fam_wife{$fam_id} = $1) =~ tr/I/i/;
      }
      if ($rol =~ /CHIL\s@(.*)@/) {
        #$key = $fam_id."@".$famc_cnt."@".$1;
        #$fam_chil{$key} = $1;
        ($tmp_id = $1) =~ tr/I/i/;
        if ($famc_cnt == 1) {   # First one
          $fam_chil{$fam_id} = $tmp_id;}
        else {
          $fam_chil{$fam_id} = $fam_chil{$fam_id}."@".$tmp_id;}
        $key = $fam_id;
        $famc_cnt++;
      }
      elsif ($rol =~ /MARR.*/) {
        $in1 = "marriage";
      }
      elsif ($rol =~ /SLGS.*/) {
        $in1 = "seal spouse";
      }
    }
    elsif ($in eq "individual_record") { 
      #print "\tindividual $indiv_id $rol\n";
      if ($rol =~ /NAME\s(.*)/) {
        # convert the surname to italics
        $name = $1;
        $name =~ /(.*)\/(.*)\/(.*)/;
        $indiv_surname{$indiv_id} = $2;
        $indiv_forname{$indiv_id} = $1." ".$3;          
        $_ = $name;
        s/\// <i>/;
        s/\//<\/i>/;
        $indiv_name{$indiv_id} = $_;
        $_ = $name;
        s/\// /g;
        $indiv_name_unformatted{$indiv_id} = $_;                
      }
      elsif ($rol =~ /SEX\s(.)/) {
        $indiv_sex{$indiv_id} = $1;
      }
      elsif ($rol =~ /BIRT.*/) {
        $in1 = "birth";
      }
      elsif ($rol =~ /DEAT.*/) {
        $in1 = "death";
      }
      elsif ($rol =~ /BURI.*/) {
        $in1 = "burial";
      }
      elsif ($rol =~ /BAPL.*/) {
        $in1 = "baptism";
      }
      elsif ($rol =~ /ENDL.*/) {
        $in1 = "endowment";
      }
      elsif ($rol =~ /SLGC.*/) {
        $in1 = "seal parents";
      }
      elsif ($rol =~ /OCCU\s(.*)/) {
        $indiv_occu{$indiv_id} = $1;
      }
      elsif ($rol =~ /NOTE\s@(.*)@/) {
        # note with link to level 0 note record
        #$indiv_note_link{$indiv_id} = $1;
        ($tmp_id = $1) =~ tr/I/i/;
        $note_indiv_link{$tmp_id} = $indiv_id;
        $in1 = "notes";
      }
      elsif ($rol =~ /NOTE\s(.*)\\$/) {
        $indiv_note{$indiv_id} = $indiv_note{$indiv_id}."<br>\n$1";
        $no_br_cont = 1;
      }
      elsif ($rol =~ /NOTE\s(.*)/) {
        $indiv_note{$indiv_id} = $1 . "<br>\n";
        $no_br_cont = 0;
        $in1 = "notes";
      }
      elsif ($rol =~ /FAMC\s@(.*)@/) { # child to family link
        ($indiv_famc{$indiv_id} = $1) =~ tr/I/i/;
      }
      elsif ($rol =~ /FAMS\s@(.*)@/) { # spouse to family link
        ($tmp_id = $1) =~ tr/I/i/;
        if ($num_fams > 0) {
          $indiv_fams{$indiv_id} = $indiv_fams{$indiv_id}."@".$tmp_id;
        }
        else {
          $indiv_fams{$indiv_id} = $tmp_id;                    
        }
        $num_fams++;
      }
    }
    elsif ($in eq "note_record") { 
      # $note_id has the note link code
      if ($rol =~ /CONT\s?(.*)\\$/) {
        if ($no_br_cont) {
          $note{$note_id} = $note{$note_id}."$1";
        } else {
          $note{$note_id} = $note{$note_id}."<br>\n$1";
        }
        $no_br_cont = 1;
      }
      elsif ($rol =~ /CONC\s?(.*)/) {
        $note{$note_id} = $note{$note_id}."$1";
      }
      elsif ($rol =~ /CONT\s?(.*)/) {
        if ($no_br_cont) {
          $note{$note_id} = $note{$note_id}."$1";
        } else {
          $note{$note_id} = $note{$note_id}."<br>\n$1";
        }
        $no_br_cont = 0;
      }
    }
  }                             # end if a 1 line
  elsif (/^\s*2\s(.*)/) {       # a 2 line
    $rol = $1;
    if ($in1 eq "birth") {
      if ($rol =~ /DATE\s(.*)/) {
        $indiv_birt_date{$indiv_id} = $1;
        $indiv_birt{$indiv_id} = 1;
      }
      elsif ($rol =~ /PLAC\s(.*)/) {
        $indiv_birt_plac{$indiv_id} = $1;
        $indiv_birt{$indiv_id} = 1;
      }
    }
    elsif ($in1 eq "baptism") {
      if ($rol =~ /DATE\s(.*)/) {
        $indiv_bap_date{$indiv_id} = $1;
        $indiv_bap{$indiv_id} = 1;
      }
      elsif ($rol =~ /PLAC\s(.*)/) {
        $indiv_bap_plac{$indiv_id} = $1;
        $indiv_bap{$indiv_id} = 1;
      }
    }
    elsif ($in1 eq "endowment") {
      if ($rol =~ /DATE\s(.*)/) {
        $indiv_end_date{$indiv_id} = $1;
        $indiv_end{$indiv_id} = 1;
      }
      elsif ($rol =~ /STAT\s(.*)/) {
        $indiv_end_plac{$indiv_id} = $1;
        $indiv_end{$indiv_id} = 1;
      }
      elsif ($rol =~ /TEMP\s(.*)/) {
        $indiv_end_plac{$indiv_id} = $1;
        $indiv_end{$indiv_id} = 1;
      }
      elsif ($rol =~ /PLAC\s(.*)/) {
        $indiv_end_plac{$indiv_id} = $1;
        $indiv_end{$indiv_id} = 1;
      }
    }
    elsif ($in1 eq "seal parents") {
      if ($rol =~ /DATE\s(.*)/) {
        $indiv_slgc_date{$indiv_id} = $1;
        $indiv_slgc{$indiv_id} = 1;
      }
      elsif ($rol =~ /STAT\s(.*)/) {
        $indiv_slgc_plac{$indiv_id} = $1;
        $indiv_slgc{$indiv_id} = 1;
      }
      elsif ($rol =~ /TEMP\s(.*)/) {
        $indiv_slgc_plac{$indiv_id} = $1;
        $indiv_slgc{$indiv_id} = 1;
      }
      elsif ($rol =~ /PLAC\s(.*)/) {
        $indiv_slgc_plac{$indiv_id} = $1;
        $indiv_slgc{$indiv_id} = 1;
      }
    }
    elsif ($in1 eq "death") {
      if ($rol =~ /DATE\s(.*)/) {
        $indiv_deat_date{$indiv_id} = $1;
        $indiv_deat{$indiv_id} = 1;
      }
      elsif ($rol =~ /PLAC\s(.*)/) {
        $indiv_deat_plac{$indiv_id} = $1;
        $indiv_deat{$indiv_id} = 1;
      }
    }
    elsif ($in1 eq "burial") {
      if ($rol =~ /DATE\s(.*)/) {
        $indiv_buri_date{$indiv_id} = $1;
        $indiv_buri{$indiv_id} = 1;
      }
      elsif ($rol =~ /PLAC\s(.*)/) {
        $indiv_buri_plac{$indiv_id} = $1;
        $indiv_buri{$indiv_id} = 1;
      }
    }
    elsif ($in1 eq "notes") {
      if ($rol =~ /CONT\s?(.*)\\$/) {
        if ($no_br_cont) {
          $indiv_note{$indiv_id} = $indiv_note{$indiv_id}."$1";
        } else {
          $indiv_note{$indiv_id} = $indiv_note{$indiv_id}."<br>\n$1";
        }
        $no_br_cont = 1;
      }
      elsif ($rol =~ /CONC\s?(.*)/) {
        $indiv_note{$indiv_id} = $indiv_note{$indiv_id}."$1";
      }
      elsif ($rol =~ /CONT\s?(.*)/) {
        if ($no_br_cont) {
          $indiv_note{$indiv_id} = $indiv_note{$indiv_id}."$1";
        } else {
          $indiv_note{$indiv_id} = $indiv_note{$indiv_id}."<br>\n$1";
        }
        $no_br_cont = 0;
      }
    }
    elsif ($in1 eq "marriage") {
      if ($rol =~ /DATE\s(.*)/) {
        $fam_marr_date{$fam_id} = $1;
        $fam_marr{$fam_id} = 1;
      }
      elsif ($rol =~ /PLAC\s(.*)/) {
        $fam_marr_plac{$fam_id} = $1;
        $fam_marr{$fam_id} = 1;
      }
    }
    elsif ($in1 eq "seal spouse") {
      if ($rol =~ /DATE\s(.*)/) {
        $fam_slgs_date{$fam_id} = $1;
        $fam_slgs{$fam_id} = 1;
      }
      elsif ($rol =~ /STAT\s(.*)/) {
        $indiv_slgs_plac{$indiv_id} = $1;
        $indiv_slgs{$indiv_id} = 1;
      }
      elsif ($rol =~ /TEMP\s(.*)/) {
        $indiv_slgs_plac{$indiv_id} = $1;
        $indiv_slgs{$indiv_id} = 1;
      }
      elsif ($rol =~ /PLAC\s(.*)/) {
        $fam_slgs_plac{$fam_id} = $1;
        $fam_slgs{$fam_id} = 1;
      }
    }
  }                             # end if a 2 line
}

#################################################################
# Link note data to individual records
foreach $note_id (keys %note) {
    $indiv_note{$note_indiv_link{$note_id}} = $indiv_note{$note_indiv_link{$note_id}}.$note{$note_id};
}    

#################################################################
# Set up the HTML for the top and bottom of individual's files
if (open(IN_FILE, "tpl_ind_top.html")) {
    $i = 0;
    while (<IN_FILE>) {
        $html_ind_top[$i] = $_;
        $i++;
    }
}
else { # use defaults
    $html_ind_top[0] = "<body onLoad = \"onLoad()\">\n";
    $html_ind_top[1] = "<hr>\n";
    $html_ind_top[2] = button_bar("grp", $indiv_id, 0);
    $html_ind_top[3] = "<hr>\n";
    $html_ind_top[4] = "<h1>#ind_name</h1>\n";    
}
close(IN_FILE);

if (open(IN_FILE, "tpl_ind_bot.html")) {
    $i = 0;
    while (<IN_FILE>) {
        $html_ind_bot[$i] = $_;
        $i++;
    }
}
else { # use defaults
    $html_ind_bot[0] = "<hr>\n";
    $html_ind_bot[1] = button_bar("grp", $indiv_id, 2);
    $html_ind_bot[2] = "<hr>\n";
}
close(IN_FILE);

#################################################################
# show results
print "\nCreating individual files\n";
$ind_cnt = 0;
# make a file for each individual
foreach $indiv_id (keys %indivs) {
  $ind_cnt++;
  
  if ($debug) {
    print DBUG_OUT "$ind_cnt\n";
  } 
  
  print "$ind_cnt\r";
  
  # Clear out $div_str and $grp_out_str
  $div_str = "";
  $grp_out_str = "";
  $switch_str = "";
  
  # open an output file
  unless (open(GRP_OUT, ">$out_dir/$indiv_id.html")) {
    die "\nCouldn't open output file $out_dir/$indiv_id.html\n";
  }
  
  # Split the list of spouse families into @fams.
  # Need to clear the array for some versions of perl.
  @fams = ();
  @fams = split(/@/, $indiv_fams{$indiv_id});
  #print "Fams:@fams\n";
  $famc = $indiv_famc{$indiv_id};
  
  ##############################################################
  # print the information to file
  # Make birth info private if required and person alive
  if ($private && $indiv_deat{$indiv_id} == 0) {
    # No death record - check if born too long ago
    if ($indiv_birt_date{$indiv_id} =~ /([0-9][0-9][0-9][0-9])/) {
      $year = $1;
      if ($year > $curr_year) {
        print "Individual $indiv_id has birth date ($1) after this year ($curr_year)\n";
      }
      elsif ($year+120 > $curr_year) { # May be alive
        &make_birt_private($indiv_id);
      }
    }
    else {                      # Couldn't find a birth year - make private
      &make_birt_private($indiv_id);
    }
  }
  
  # Check if there is a phot for the individual
  # Thanks to Bob Minteer for the code this is based on
  if ($check_images) {
    if (-r "$out_dir/$photo_dir/$indiv_id.jpg") {
      $imgpath{$indiv_id}="$photo_dir/$indiv_id.jpg";
    }
    elsif (-r "$out_dir/$photo_dir/$indiv_id.gif") {
      $imgpath{$indiv_id}="$photo_dir/$indiv_id.gif";
    }
    else {
      $imgpath{$indiv_id}="";
    }
  }
  
  # print HTML at top of page
#  $html_ind_top_l = @html_ind_top;
#  for ($i = 0; $i < $html_ind_top_l; $i++) {
#    $_ = $html_ind_top[$i];
#    s/\#ind_name/$indiv_name{$indiv_id}/ei;
#    $grp_out_str .= $_;
#    if (($check_images) && ($imgpath{$indiv_id} ne "") && (/<body/)) {
#
#  $grp_out_str .= "<IMG SRC=\"$imgpath{$indiv_id}\" ALIGN=\"right\">\n";
#    }
#  }
  $grp_out_str .= "<body onLoad = \"onLoad()\">\n";
  $grp_out_str .= "<hr>\n";
  $grp_out_str .= button_bar("grp", $indiv_id, 0);
  $grp_out_str .= "<hr>\n";
  $grp_out_str .= "<h1>$indiv_name{$indiv_id}</h1>\n";    
  
  $grp_out_str .= &create_vital_info($indiv_id);
  
  # print marriage info if more than just spouse name is known
  if (!$print_family) {
    &print_marriages;
  }
  
  if ($family_table || $print_family) {
    &get_parent_data;
    &get_child_data;
  }
  
  # If $family_table then print the ancestors chart (needs $father,
  # $mother, $pgfather etc., $famc.
  if ($family_table) {
    &get_gparent_data;
    
    # family chart
    $grp_out_str .= "<p>\n<table border=1 cellspacing=0 cellpadding=0>\n";
    if (($father ne "") || ($mother ne "")) {
      # Ancestors            
      $grp_out_str .= "<tr><td align=center><table border=0></td>\n";
      $grp_out_str .= "<tr>\n";
      
      # Paternal Grandfather
      if ($pgfather_id ne "") {
        $grp_out_str .= "<td width=150 align=center><a href=\"$pgfather_id.html\" onMouseOver = \"show('$pgfather_id')\" onMouseOut = \"hide('$pgfather_id')\">$pgfather</a></td>\n";
        add_div($pgfather_id);
        add_switch($pgfather_id);
      } else {
        $grp_out_str .= "<td width=150 align=center></td>\n";
      }
      
      # Paternal Grandmother
      if ($pgmother_id ne "") {
        $grp_out_str .= "<td width=150 align=center><a href=\"$pgmother_id.html\" onMouseOver = \"show('$pgmother_id')\" onMouseOut = \"hide('$pgmother_id')\">$pgmother</a></td>\n";
        add_div($pgmother_id);
        add_switch($pgmother_id);
      } else {
        $grp_out_str .= "<td width=150 align=center></td>\n";
      }
      
      # Maternal Grandfather
      if ($mgfather_id ne "") {
        $grp_out_str .= "<td width=150 align=center><a href=\"$mgfather_id.html\" onMouseOver = \"show('$mgfather_id')\" onMouseOut = \"hide('$mgfather_id')\">$mgfather</a></td>\n";
        add_div($mgfather_id);
        add_switch($mgfather_id);
      } else {
        $grp_out_str .= "<td width=150 align=center></td>\n";
      }
      
      # Maternal Grandmother
      if ($mgmother_id ne "") {
        $grp_out_str .= "<td width=150 align=center><a href=\"$mgmother_id.html\" onMouseOver = \"show('$mgmother_id')\" onMouseOut = \"hide('$mgmother_id')\">$mgmother</a></td>\n";
        add_div($mgmother_id);
        add_switch($mgmother_id);
      } else {
        $grp_out_str .= "<td width=150 align=center></td>\n";
      }
      
      $grp_out_str .= "</tr>\n<tr>\n";
      if (($pgfather ne "") || ($pgmother ne ""))
        {$grp_out_str .= "<td colspan=2><img src=\"$treepic_path/tree300.gif\" width=\"300\" height=\"30\"></td>\n";}
      else
        {$grp_out_str .= "<td colspan=2></td>\n";}
      if (($mgfather ne "") || ($mgmother ne ""))
        {$grp_out_str .= "<td colspan=2><img src=\"$treepic_path/tree300.gif\" width=\"300\" height=\"30\"></td>\n";}
      else
        {$grp_out_str .= "<td colspan=2></td>\n";}
      $grp_out_str .= "</tr>\n";
      $grp_out_str .= "<tr>\n";
      
      # Father
      $grp_out_str .= "<td colspan=2 align=center><a href=\"$father_id.html\" onMouseOver = \"show('$father_id')\" onMouseOut = \"hide('$father_id')\">$father</a></td>\n";
      add_div($father_id);
      add_switch($father_id);
      
      # Mother
      $grp_out_str .= "<td colspan=2 align=center><a href=\"$mother_id.html\" onMouseOver = \"show('$mother_id')\" onMouseOut = \"hide('$mother_id')\">$mother</a></td>\n";
      add_div($mother_id);
      add_switch($mother_id);
      
      $grp_out_str .= "</tr>\n";
      $grp_out_str .= "<tr>\n";
      if (($father ne "") || ($mother ne "")) 
        {$grp_out_str .= "<td colspan=4><img src=\"$treepic_path/tree600.gif\" width=\"600\" height=\"30\"></td>\n";}
      else
        {$grp_out_str .= "<td colspan=4></td>\n";}
      $grp_out_str .= "<tr>\n";
      $grp_out_str .= "<th colspan=4 align=center>$indiv_name{$indiv_id}</th>\n";
      $grp_out_str .= "</tr>\n";
      $grp_out_str .= "</table>\n</tr>\n<p>\n";
    }                           # end if has a parent
    $fams_len = @fams;
    
    if ($fams_len != 0) {
      # Spouse(s) and their children
      for ($fams_num=0; $fams_num < $fams_len; $fams_num++) {
        &get_spouse_data($fams[$fams_num]);
        &get_child_data($fams[$fams_num]);
        if ($num_children > 0) {
          $grp_out_str .= "\n<tr><td align=center>\n<table border=0 cellspacing=0 cellpadding=0></td>\n";
          $grp_out_str .= "<tr>\n";
          if ($num_children > 8) {
            $cwidth=9;
          }
          else {
            $cwidth = $num_children;
          }
          $grp_out_str .= "<td colspan=$cwidth align=center>";
          $grp_out_str .= $str_m;
          if ($fams_len > 1) {
            $i = $fams_num+1;
            $grp_out_str .= "($i) ";
          }
          
          # Spouse
          $grp_out_str .= "<a href=\"$spouse_id.html\" onMouseOver = \"show('$spouse_id')\" onMouseOut = \"hide('$spouse_id')\">$spouse</a></td></tr>\n";
          add_div($spouse_id);
          add_switch($spouse_id);
          
          # the children
          $start_child = 0;
          while (($num_children - $start_child) > 8) {
            # print the 8 child tree with extra leg coming down
            $grp_out_str .= "<tr><td colspan=9><img src=\"$treepic_path/tree_c8l.gif\" width=600 height=30></td></tr>\n";
            # print the first 8 children
            $cell_width = 600/9;
            $grp_out_str .= "<tr>";
            for ($i = $start_child; $i < $start_child+4; $i++) {
              $grp_out_str .= "<td align=center width=67>";
              $grp_out_str .= "<img src=\"$treepic_path/dot-trans.gif\" width=67 height=1><br>";
              $grp_out_str .= "<a href=\"$child[$i].html\" onMouseOver = \"show('$child[$i]')\" onMouseOut = \"hide('$child[$i]')\"><font size=-1>$indiv_name{$child[$i]}</font></a></td>\n";
              add_div($child[$i]);
              add_switch($child[$i]);
            }
            $grp_out_str .= "<td align=center width=64><img src=\"$treepic_path/tree_l.gif\" width=64 height=60></td>\n";                        
            for ($i = $start_child+4; $i < $start_child+8; $i++) {
              $grp_out_str .= "<td align=center width=67>";
              $grp_out_str .= "<img src=\"$treepic_path/dot-trans.gif\" width=67 height=1><br>";
              $grp_out_str .= "<a href=\"$child[$i].html\" onMouseOver = \"show('$child[$i]')\" onMouseOut = \"hide('$child[$i]')\"><font size=-1>$indiv_name{$child[$i]}</font></a></td>\n";
              add_div($child[$i]);
              add_switch($child[$i]);
            }
            $grp_out_str .= "</tr>\n";
            $start_child += 8;
          }
          # print the rest of the children
          $num_left = $num_children-$start_child;
          $grp_out_str .= "<tr><td align=center colspan=9>\n";
          $grp_out_str .= "<table border=0 cellspacing=0 cellpadding=0></td>";
          $grp_out_str .= "<tr><td colspan=$num_left><img src=\"$treepic_path/tree_c$num_left.gif\" width=600 height=30></td></tr>\n";
          $cell_width = int(600/$num_left);
          $grp_out_str .= "<tr>";
          for ($i = $start_child; $i < $num_children; $i++) {
            $grp_out_str .= "<td align=center width=$cell_width>";
            $grp_out_str .= "<img src=\"$treepic_path/dot-trans.gif\" width=$cell_width height=1><br>";
            $grp_out_str .= "<a href=\"$child[$i].html\" onMouseOver = \"show('$child[$i]')\" onMouseOut = \"hide('$child[$i]')\"><font size=-1>$indiv_name{$child[$i]}</font></a><p></td>\n";
            add_div($child[$i]);
            add_switch($child[$i]);
          }
          $grp_out_str .= "</tr></table></td></tr>\n";
          
          $grp_out_str .= "</table>\n</td></tr>\n";
        }                       # end if has children
        else {                  #no children so just put marriage for consistency
          $grp_out_str .= "<tr>\n";
          $grp_out_str .= "<td align=center>";
          $grp_out_str .= $str_m;
          if ($fams_len > 1) {
            $i = $fams_num+1;
            $grp_out_str .= "($i) ";
          }
          $grp_out_str .= "<a href=\"$spouse_id.html\" onMouseOver = \"show('$spouse_id')\" onMouseOut = \"hide('$spouse_id')\">$spouse</a><p></td></tr>\n";   
          add_div($spouse_id);
          add_switch($spouse_id);
        }
      }                         # end for each family
    }                           # end if has spouses
    $grp_out_str .= "</table>\n<p>\n";
    
  }                             # end if family_table
  
  # If $print_family then print parents, marriage(s), children.
  # Uses $father, $mother, @child, $famc, $fams.
  
  if ($print_family) {
    
    # parents
    if ($father ne "") {
      $grp_out_str .= "$str_father <a href=\"$father_id.html\" onMouseOver = \"show('$father_id')\" onMouseOut = \"hide('$father_id')\">$father</a><br>\n"; 
      add_div($father_id);
      add_switch($father_id);
    }
    if ($mother ne "") {
      $grp_out_str .= "$str_mother <a href=\"$mother_id.html\" onMouseOver = \"show('$mother_id')\" onMouseOut = \"hide('$mother_id')\">$mother</a><p>\n"; 
      add_div($mother_id);
      add_switch($mother_id);
    }    
    $grp_out_str .= "<p>\n";
    
    # spouse, marriage date and place and children
    &print_marriages(1);
    
  }                             # end if print_family
  
  $grp_out_str .= "<p>\n";
  
  # notes
  if ($print_notes && $indiv_note{$indiv_id}) {

      $grp_out_str .= "<hr>\n";
      $grp_out_str .= button_bar("grp", $indiv_id, 1);
      $grp_out_str .= "<hr>\n";
  
      $grp_out_str .= "<h3>$str_notes</h3>\n";
      @note_lines = split (/\n/, $indiv_note{$indiv_id});
      foreach $note_line (@note_lines) {
          $grp_out_str .= get_note_link($note_line) . "\n";
      }
      $grp_out_str .= "<p>\n";
  }
  
  # Print out Div constructs
  $grp_out_str .= $div_str;
  
  print GRP_OUT "<html>\n";
  print GRP_OUT "<head id='id' data='$indiv_id'>\n";
  print GRP_OUT "<title>\n";
  print GRP_OUT "$indiv_name_unformatted{$indiv_id} \n";
  print GRP_OUT "($indiv_birt_date{$indiv_id} - \n";
  print GRP_OUT "$indiv_deat_date{$indiv_id})\n";
  print GRP_OUT "</title>\n";
  print GRP_OUT "<SCRIPT LANGUAGE=\"JavaScript1.2\">\n";
  print GRP_OUT "<!--\n";
  print GRP_OUT "ns4 = (document.layers)? true:false\n";
  print GRP_OUT "ie4 = (document.all)? true:false\n";
  print GRP_OUT "use_forms = (top.fframe)? true:false\n";

  # Pre-load button images
  print GRP_OUT "arImageSrc = new Array (\n";
  print GRP_OUT "\"surname_dn.gif\",\n";
  print GRP_OUT "\"surname_up.gif\",\n";
  print GRP_OUT "\"surname_act.gif\",\n";
  print GRP_OUT "\"people_dn.gif\",\n";
  print GRP_OUT "\"people_up.gif\",\n";
  print GRP_OUT "\"people_act.gif\",\n";
  print GRP_OUT "\"pedigree_dn.gif\",\n";
  print GRP_OUT "\"pedigree_up.gif\",\n";
  print GRP_OUT "\"pedigree_act.gif\"\n";
  print GRP_OUT ")\n";
  print GRP_OUT "arImageList = new Array ();\n";
  print GRP_OUT "for (counter in arImageSrc) {\n";
  print GRP_OUT "arImageList[counter] = new Image();\n";
  print GRP_OUT "arImageList[counter].src = \"$gif_path/\" + \n";
  print GRP_OUT "arImageSrc[counter];\n";
  print GRP_OUT "}\n";

  print GRP_OUT "function show(id) {\n";
  print GRP_OUT "if (use_forms) {\n";
  print GRP_OUT "  switch (id) {\n";
  print GRP_OUT $switch_str;
  print GRP_OUT "}}\n";
  print GRP_OUT "else if (ns4) document.layers[id].visibility = \"show\"\n";
  print GRP_OUT "else if (ie4) document.all[id].style.visibility = \"visible\"\n";
  print GRP_OUT "}\n";
  print GRP_OUT "function hide(id) {\n";
  print GRP_OUT "if (use_forms) {\n";
  print GRP_OUT "top.fframe.document.vform.birthd.value = \"\"\n";
  print GRP_OUT "top.fframe.document.vform.birthp.value = \"\"\n";
  print GRP_OUT "top.fframe.document.vform.deathd.value = \"\"\n";
  print GRP_OUT "top.fframe.document.vform.deathp.value = \"\"\n";
  print GRP_OUT "top.fframe.document.vform.buriald.value = \"\"\n";
  print GRP_OUT "top.fframe.document.vform.burialp.value = \"\"\n";

  if ($lds_ord) {
    print GRP_OUT "top.fframe.document.vform.bapd.value = \"\"\n";
    print GRP_OUT "top.fframe.document.vform.bapp.value = \"\"\n";
    print GRP_OUT "top.fframe.document.vform.endd.value = \"\"\n";
    print GRP_OUT "top.fframe.document.vform.endp.value = \"\"\n";
    print GRP_OUT "top.fframe.document.vform.slgcd.value = \"\"\n";
    print GRP_OUT "top.fframe.document.vform.slgcp.value = \"\"\n";
  }

  print GRP_OUT "}\n";
  print GRP_OUT "else if (ns4) document.layers[id].visibility = \"hide\"\n";
  print GRP_OUT "else if (ie4) document.all[id].style.visibility = \"hidden\"\n";
  print GRP_OUT "}\n";
  print GRP_OUT "function onLoad() {\n";
  print GRP_OUT "if (use_forms) {\n";
  print GRP_OUT "top.fframe.document.vform.birthd.value = \"\"\n";
  print GRP_OUT "top.fframe.document.vform.birthp.value = \"\"\n";
  print GRP_OUT "top.fframe.document.vform.deathd.value = \"\"\n";
  print GRP_OUT "top.fframe.document.vform.deathp.value = \"\"\n";
  print GRP_OUT "top.fframe.document.vform.buriald.value = \"\"\n";
  print GRP_OUT "top.fframe.document.vform.burialp.value = \"\"\n";

  if ($lds_ord) {
    print GRP_OUT "top.fframe.document.vform.bapd.value = \"\"\n";
    print GRP_OUT "top.fframe.document.vform.bapp.value = \"\"\n";
    print GRP_OUT "top.fframe.document.vform.endd.value = \"\"\n";
    print GRP_OUT "top.fframe.document.vform.endp.value = \"\"\n";
    print GRP_OUT "top.fframe.document.vform.slgcd.value = \"\"\n";
    print GRP_OUT "top.fframe.document.vform.slgcp.value = \"\"\n";
  }

  print GRP_OUT "}\n";
  print GRP_OUT "}\n";
  print GRP_OUT "//-->\n";
  print GRP_OUT "</SCRIPT>\n";
  print GRP_OUT "</head>\n\n";
  
  print GRP_OUT $grp_out_str;
  
  # print HTML at bottom of page
#  print GRP_OUT @html_ind_bot;
  print GRP_OUT "<hr>\n";  
  print GRP_OUT button_bar("grp", $indiv_id, 2);
  print GRP_OUT "<hr>\n";  
  #    print GRP_OUT "</BODY>\n";
  print GRP_OUT &print_footer;
  close(GRP_OUT);
  
  # Create Pedigree Chart
  
  # Clear out $div_str and $ped_out_str
  $div_str = "";
  $ped_out_str = "";
  $switch_str = "";
  
  unless (open(PED_OUT, ">$out_dir/p$indiv_id.html")) {
    die "\nCouldn't open output file $out_dir/p$indiv_id.html\n";
  }
  
  ($bogus, $ped_depth) = ped_parse ($indiv_id, 1, 1);
  
  # Print Generation Header
  $ped_out_str .= "<TR>";
  
  if ($debug) {
    print DBUG_OUT "ped_depth = $ped_depth\n";
  }
  
  for ($cnt = 1; $cnt <= $ped_depth; $cnt++) {
    $ped_out_str .= "<TH>$cnt</TH>";
  }
  
  $ped_out_str .= "\n";
  
  ped_print (1, 1, 0, 0, 1);
  
  $ped_out_str .= "</TABLE>\n";
  $ped_out_str .= "</TABLE>\n";
  
  $ped_out_str .= "<hr>\n";
  $ped_out_str .= button_bar("ped", $indiv_id, 1);
  $ped_out_str .= "<hr>\n";
  
  # Print out Div constructs
  $ped_out_str .= $div_str;
  
  $ped_out_str .= "<P>\n";
  
  # Print to file
  print PED_OUT "<head id='id' data='p$indiv_id'>\n";
  print PED_OUT "<title>\n";
  print PED_OUT "$indiv_name_unformatted{$indiv_id} \n";
  print PED_OUT "($indiv_birt_date{$indiv_id} - \n";
  print PED_OUT "$indiv_deat_date{$indiv_id})\n";
  print PED_OUT "</title>\n";
  print PED_OUT "<SCRIPT LANGUAGE=\"JavaScript1.2\">\n";
  print PED_OUT "<!--\n";
  print PED_OUT "ns4 = (document.layers)? true:false\n";
  print PED_OUT "ie4 = (document.all)? true:false\n";
  print PED_OUT "use_forms = (top.fframe)? true:false\n";

  print PED_OUT "arImageSrc = new Array (\n";
  print PED_OUT "\"surname_dn.gif\",\n";
  print PED_OUT "\"surname_up.gif\",\n";
  print PED_OUT "\"surname_act.gif\",\n";
  print PED_OUT "\"people_dn.gif\",\n";
  print PED_OUT "\"people_up.gif\",\n";
  print PED_OUT "\"people_act.gif\",\n";
  print PED_OUT "\"group_dn.gif\",\n";
  print PED_OUT "\"group_up.gif\",\n";
  print PED_OUT "\"group_act.gif\"\n";
  print PED_OUT ")\n";
  print PED_OUT "arImageList = new Array ();\n";
  print PED_OUT "for (counter in arImageSrc) {\n";
  print PED_OUT "arImageList[counter] = new Image();\n";
  print PED_OUT "arImageList[counter].src = \"$gif_path/\" + \n";
  print PED_OUT "arImageSrc[counter];\n";
  print PED_OUT "}\n";

  print PED_OUT "function show(id) {\n";
  print PED_OUT "if (use_forms) {\n";
  print PED_OUT "  switch (id) {\n";
  print PED_OUT $switch_str;
  print PED_OUT "}}\n";
  print PED_OUT "else if (ns4) document.layers[id].visibility = \"show\"\n";
  print PED_OUT "else if (ie4) document.all[id].style.visibility = \"visible\"\n";
  print PED_OUT "}\n";
  print PED_OUT "function hide(id) {\n";
  print PED_OUT "if (use_forms) {\n";
  print PED_OUT "top.fframe.document.vform.birthd.value = \"\"\n";
  print PED_OUT "top.fframe.document.vform.birthp.value = \"\"\n";
  print PED_OUT "top.fframe.document.vform.deathd.value = \"\"\n";
  print PED_OUT "top.fframe.document.vform.deathp.value = \"\"\n";
  print PED_OUT "top.fframe.document.vform.buriald.value = \"\"\n";
  print PED_OUT "top.fframe.document.vform.burialp.value = \"\"\n";

  if ($lds_ord) {
    print PED_OUT "top.fframe.document.vform.bapd.value = \"\"\n";
    print PED_OUT "top.fframe.document.vform.bapp.value = \"\"\n";
    print PED_OUT "top.fframe.document.vform.endd.value = \"\"\n";
    print PED_OUT "top.fframe.document.vform.endp.value = \"\"\n";
    print PED_OUT "top.fframe.document.vform.slgcd.value = \"\"\n";
    print PED_OUT "top.fframe.document.vform.slgcp.value = \"\"\n";
  }

  print PED_OUT "}\n";
  print PED_OUT "else if (ns4) document.layers[id].visibility = \"hide\"\n";
  print PED_OUT "else if (ie4) document.all[id].style.visibility = \"hidden\"\n";
  print PED_OUT "}\n";
  print PED_OUT "function onLoad() {\n";
  print PED_OUT "if (use_forms) {\n";
  print PED_OUT "top.fframe.document.vform.birthd.value = \"\"\n";
  print PED_OUT "top.fframe.document.vform.birthp.value = \"\"\n";
  print PED_OUT "top.fframe.document.vform.deathd.value = \"\"\n";
  print PED_OUT "top.fframe.document.vform.deathp.value = \"\"\n";
  print PED_OUT "top.fframe.document.vform.buriald.value = \"\"\n";
  print PED_OUT "top.fframe.document.vform.burialp.value = \"\"\n";

  if ($lds_ord) {
    print PED_OUT "top.fframe.document.vform.bapd.value = \"\"\n";
    print PED_OUT "top.fframe.document.vform.bapp.value = \"\"\n";
    print PED_OUT "top.fframe.document.vform.endd.value = \"\"\n";
    print PED_OUT "top.fframe.document.vform.endp.value = \"\"\n";
    print PED_OUT "top.fframe.document.vform.slgcd.value = \"\"\n";
    print PED_OUT "top.fframe.document.vform.slgcp.value = \"\"\n";
  }

  print PED_OUT "}\n";
  print PED_OUT "}\n";
  print PED_OUT "//-->\n";
  print PED_OUT "</SCRIPT>\n";
  print PED_OUT "</head>\n\n";
  print PED_OUT "<BODY onLoad = \"onLoad()\">\n";

  print PED_OUT "<hr>\n";
  print PED_OUT button_bar("ped", $indiv_id, 0);
  print PED_OUT "<hr>\n";

  print PED_OUT "<TABLE border=2 cellpadding=5>\n";
  print PED_OUT "<TR><TD>\n";
  print PED_OUT "<TABLE border=0>\n";
  
  print PED_OUT $ped_out_str;
  
  print PED_OUT "</TABLE>\n";
  print PED_OUT "</TABLE>\n";
  #    print PED_OUT "</BODY>\n";
  
  print PED_OUT &print_footer;
  close PED_OUT;
  
  
}                               # end foreach $indiv_id (keys %indivs)


#############################################
# make a list of people file
# open an output file
print "\rCreating people file\n";
unless (open(PPL_OUT, ">$out_dir/people.html")) {
    die "Couldn't open output file $out_dir/people.html\n";
}

if ($create_gendex) {
    unless (open(GENDEX, ">gendex.txt")) {
        die "Couldn't open output file gendex.txt\n";
    }
}

if (open(TPL_FILE, "tpl_people.html")) {
    print "Using people template file\n";
    $i = 0;
    while (<TPL_FILE>) {
        $fline[$i] = $_;
        $i++;
    } # while <TPL_FILE>
    close(TPL_FILE);
}
else {
  # use defaults
  @fline = (
            "<html><head><title>$str_people</title></head>\n\n<body>\n",
            "<hr>\n",
            button_bar("ppl", 0, 0),
            "<hr>\n",
            "<h1>$str_lpeople</h1> ($num_indivs $str_people_and $num_families $str_unique_names)<p>\n",
            "#main",
            #        "#ns <a name=\"#ind_surname\">",
            "<a href=\"#ind_id.html\">#ind_surname, #ind_forname </a>(#ind_birt_date - #ind_deat_date) #photo<br>",
            "#end",
            "<p><hr>\n",
            button_bar("ppl", 0, 1),
            "<hr>\n",
            "#footer"
           );
}
$fline = @fline;
    for ($j = 0; $j < $fline; $j++) {
        $_ = $fline[$j];
        if (/#main/) { # do the main loop of all people
            # read in the format
            $i = 0; $nsi = 0;
            until ($line =~ /#end/) {
                   $j++;
                   $line = $fline[$j];
                   $ns[$i] = 0;
                   if ($line =~ /#ns(.*\n)/) {
#                  if ($line =~ /#ns(.*)/) {
                       $line = $1;
                       $ns[$i] = 1;   
                   }
                   $out_fmt[$i] = $line;
                   $i++;
            }
            $out_fmt[$i-1]=""; # get rid of #end
            $len = @out_fmt;
            $old_surname = "-o-o-";
            foreach $indiv_id (sort by_surname keys %indivs) {
                $ns_this = (($indiv_surname{$indiv_id} ne $old_surname) && ($indiv_surname{$indiv_id} ne ""));
                if ($ns_this) 
                    {$old_surname = $indiv_surname{$indiv_id};}
                for ($i = 0; $i < $len; $i++) {
                    if ($ns_this || !$ns[$i]) { # print new sentence lines only when this is
                        $line = $out_fmt[$i];
                        $line =~ s/#ind_id/$indiv_id/gei;
                        $line =~ s/#ind_forname/$indiv_forname{$indiv_id}/gei;
                        $line =~ s/#ind_surname/$indiv_surname{$indiv_id}/gei;
                        $line =~ s/#ind_birt_date/$indiv_birt_date{$indiv_id}/gei;
                        $line =~ s/#ind_deat_date/$indiv_deat_date{$indiv_id}/gei;
                        if (($check_images) && ($imgpath{$indiv_id} ne "")) {
                            $line =~ s/#photo/\<em\>\(photo available\)\<\/em\>/gi;
                        }
                        else {
                            $line =~ s/#photo//gei;
                        }
                        # get rid of private birth date
                        $line =~ s/\(\($str_private\) - \)/$str_private/;
                        print PPL_OUT $line;
                    }
                }

                if ($create_gendex) {
                    print GENDEX "$indiv_id.html|";
                    print GENDEX "$indiv_surname{$indiv_id}|";
                    print GENDEX "$indiv_forname{$indiv_id}\/$indiv_surname{$indiv_id}\/|";
                    print GENDEX "$indiv_birt_date{$indiv_id}|";
                    print GENDEX "$indiv_birt_plac{$indiv_id}|";
                    print GENDEX "$indiv_deat_date{$indiv_id}|";
                    print GENDEX "$indiv_deat_plac{$indiv_id}|\n";
                }
            } # end for each individual
            @out_fmt = ();
        }
        elsif (/#footer/) {
            print PPL_OUT &print_footer;
        }
        else {
            print PPL_OUT $_;
        }
    } # for i=0 to $fline
close(PPL_OUT);
close(GENDEX) if ($create_gendex);


#############################################
# make a surname file
unless (open(SUR_OUT, ">$out_dir/surnames.html")) {
    die "Couldn't open output file $out_dir/surnames.html\n";
}

print "Creating surnames file\n";
if (open(TPL_FILE, "tpl_surnames.html")) {
    print "Using surnames template file\n";
    while (<TPL_FILE>) {
        if (/#main/) { # do the main loop of all surnames
            # read in the format
            $i = 0;
            until ($line =~ /#end/) {
                $line = <TPL_FILE>;
                $out_fmt[$i] = $line;
                $i++;
            }
            $out_fmt[$i-1]=""; # get rid of #end
            $len = @out_fmt;
            $old_surname="-o-o-"; # so that empty surnames show up
            foreach $indiv_id (sort by_surname keys %indiv_surname) {
                if (lc($indiv_surname{$indiv_id}) ne $old_surname) {
                    for ($i = 0; $i < $len; $i++) {
                        $line = $out_fmt[$i];
                        $line =~ s/#ind_surname/$indiv_surname{$indiv_id}/gei;
                        print SUR_OUT $line;
                    }
                    $old_surname = lc($indiv_surname{$indiv_id});
                }
            } # end for each individual
            @out_fmt = ();
        }
        elsif (/#footer/) {
            print SUR_OUT &print_footer;
        }
        else {
            print SUR_OUT $_;
        }
    } # while <TPL_FILE>
    close(TPL_FILE);
}
else { # use the default layout
    print SUR_OUT "<html><head><title>Surnames</title></head>\n\n";
    print SUR_OUT "<body>\n";
    print SUR_OUT "<hr>\n";
    print SUR_OUT button_bar("sur", 0, 0);
    print SUR_OUT "<hr>\n";
    print SUR_OUT "<h1>$str_all_surnames</h1>\n";
    print SUR_OUT "$str_list_of $num_indivs $str_people_and $num_families $str_unique_names<p>\n";
    $old_surname="-o-o-"; # so that empty surnames show up
    foreach $indiv_id (sort by_surname keys %indiv_surname) {
        if (lc($indiv_surname{$indiv_id}) ne $old_surname) {
            print SUR_OUT "<a href=\"people.html#$indiv_surname{$indiv_id}\">";
            print SUR_OUT "$indiv_surname{$indiv_id}</a><br>\n";
            $old_surname = lc($indiv_surname{$indiv_id});
        }
    }
    
    print SUR_OUT "<hr>\n";
    print SUR_OUT button_bar("sur", 0, 1);
    print SUR_OUT "<hr>\n";
    print SUR_OUT &print_footer;
}
close(SUR_OUT);

#############################################
# make toplevel frame file
unless (open(FRAME_OUT, ">$out_dir/$top_frame_name")) {
    die "Couldn't open output file $out_dir/$top_frame_name\n";
}

print "Creating toplevel frame file, $top_frame_name\n";

print FRAME_OUT "<frameset cols=\"200,*\">\n";
print FRAME_OUT "<frame src=\"$top_form_name\" ";
print FRAME_OUT "name=\"fframe\" id=\"fframe\">\n";
print FRAME_OUT "<frame src=\"pi$root_id.html\" name=\"dframe\">\n";
print FRAME_OUT "</frameset>\n";

close(FRAME_OUT);

#############################################
# make toplevel form file
unless (open(FORM_OUT, ">$out_dir/$top_form_name")) {
    die "Couldn't open output file $out_dir/$top_form_name\n";
}

print "Creating toplevel form file, $top_form_name\n";

print FORM_OUT "<FORM name=\"vform\" id=\"vform\">\n";
print FORM_OUT "<B>Birth:</B> <BR>\n";
print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"birthd\"><BR>\n";
print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"birthp\"><BR>\n";
print FORM_OUT "</P>\n";
print FORM_OUT "<P>\n";
print FORM_OUT "<B>Death</B>: <BR>\n";
print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"deathd\"><BR>\n";
print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"deathp\"><BR>\n";
print FORM_OUT "</P>\n";
print FORM_OUT "<P>\n";
print FORM_OUT "<B>Burial</B>: <BR>\n";
print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"buriald\"><BR>\n";
print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"burialp\"><BR>\n";
print FORM_OUT "</P>\n";
print FORM_OUT "<P>\n";

if ($lds_ord) {
    print FORM_OUT "<B>Baptism:</B> <BR>\n";
    print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"bapd\"><BR>\n";
    print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"bapp\"><BR>\n";
    print FORM_OUT "</P>\n";
    print FORM_OUT "<P>\n";
    print FORM_OUT "<B>Endowment:</B> <BR>\n";
    print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"endd\"><BR>\n";
    print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"endp\"><BR>\n";
    print FORM_OUT "</P>\n";
    print FORM_OUT "<P>\n";
    print FORM_OUT "<B>Sealing Parents:</B> <BR>\n";
    print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"slgcd\"><BR>\n";
    print FORM_OUT "<INPUT readonly type=\"text\" size=200 name=\"slgcp\"><BR>\n";
    print FORM_OUT "</P>\n";
}

print FORM_OUT "</FORM>\n";

close(FORM_OUT);

#############################################
# sort by surname 
sub by_surname {
    $lca = lc($indiv_surname{$a});
    $lcb = lc($indiv_surname{$b});
    $lca cmp $lcb
        ||
    $indiv_forname{$a} cmp $indiv_forname{$b}
        ||
    $indiv_birt_date{$a} cmp $indiv_birt_date{$b}
        ||
    $a cmp $b;
}

#############################################
# Show the status of reading the file
# In: $num_indivs,  $num_families
sub show_reading_status {
    print "$num_indivs individuals, $num_families families\r";
}

#############################################
# Uses OUT_FILE.
sub print_footer {
    my $ft_str = "Created by Paul Rawlins' ";
    my $gmtime = gmtime;

    $ft_str .= "<a href=\"http://www.rawlins.org/Pedigree/WebbedGed.html\" ";
    $ft_str .= "target=\"_top\">";
    $ft_str .= "<b><i>WebbedGed</i></b> Gedcom to HTML converter</a> \n";
    $ft_str .= "v$version.<br>\n";
    $ft_str .= "Generated from ";
    if ($add_gedcom_link) {
        $ft_str .= "<a href=\"$gedcom_link_path/$gedfile$gedcom_link_ext\">\n";
        $ft_str .= "$gedfile$gedcom_link_ext</a> ";
    }
    else {
        $ft_str .= "$gedfile$gedcom_link_ext ";
    }
    $ft_str .= "at $gmtime GMT<br>\n";
    $ft_str .= "</body></html>\n";
    return $ft_str;
}

#############################################
# Uses OUT_FILE, $spouse, $spouse_id, $fams.
sub print_marriages {
    $long = $_[0]; # if set then print children too
    $fams_len = @fams;
    for ($fams_num=0; $fams_num < $fams_len; $fams_num++) {
        &get_spouse_data($fams[$fams_num]);
        if (($spouse ne "") || $fam_marr{$fams[$fams_num]}) {
            $grp_out_str .= "<dl><dt><b>$str_married </b>";
            if ($fams_len > 1) {
                $i = $fams_num+1;
                $grp_out_str .= "($i) ";
            }
        }
        if ($spouse ne "") {
            # Spouse
            $grp_out_str .= "<a href=\"$spouse_id.html\" onMouseOver = \"show('$spouse_id')\" onMouseOut = \"hide('$spouse_id')\">$spouse</a>";
        }
        if ($fam_marr{$fams[$fams_num]}) {
            if ($fam_marr_date{$fams[$fams_num]}) {
                $grp_out_str .= " $str_on$fam_marr_date{$fams[$fams_num]}";
            }
            if ($fam_marr_plac{$fams[$fams_num]}) {
                $grp_out_str .= " $str_at$fam_marr_plac{$fams[$fams_num]}";
            }
        }
        $grp_out_str .= "\n";
        if ($lds_ord && $fam_slgs{$fams[$fams_num]}) {
            $grp_out_str .= "<br><b>$str_slgs </b>";
            if ($fam_slgs_date{$fams[$fams_num]}) {
                $grp_out_str .= " $str_on$fam_slgs_date{$fams[$fams_num]}";
            }
            if ($fam_slgs_plac{$fams[$fams_num]}) {
                $grp_out_str .= " $str_at$fam_slgs_plac{$fams[$fams_num]}";
            }
            $grp_out_str .= "\n";
        }
        if ($long) {
            &get_child_data($fams[$fams_num]);
            # children (in order of fam entry)
            for ($i = 0; $i < $num_children; $i++) {

                # Child
                $child_num = $i+1;
                $grp_out_str .= "<dd>$str_child $child_num: <a href=\"$child[$i].html\" onMouseOver = \"show('$child[$i]')\" onMouseOut = \"hide('$child[$i]')\">$indiv_name{$child[$i]}</a><br>\n";
                add_div($child[$i]);
                add_switch($child[$i]);
            }
        }
        $grp_out_str .= "</dl><p>\n";
    }
}

#############################################
# Sets $father_id, $father, $mother_id, $mother.
# Uses $famc.
sub get_parent_data {
    # get parents
    $father = "";
    $mother = "";
    $father_id = $fam_husb{$famc};
    $mother_id = $fam_wife{$famc};
    $father = $indiv_name{$father_id};
    $mother = $indiv_name{$mother_id};
}

#############################################
# Sets $pgfather_id, $pgfather, $pgmother_id, $pgmother etc.
# Uses $father_id, $mother_id.
sub get_gparent_data {
    $pgfather = "";
    $pgmother = "";
    $mgfather = "";
    $mgmother = "";
    if ($father ne "") {
        $pgfather_id = $fam_husb{$indiv_famc{$father_id}};
        $pgfather = $indiv_name{$pgfather_id};
        $pgmother_id = $fam_wife{$indiv_famc{$father_id}};
        $pgmother = $indiv_name{$pgmother_id};
    }
    if ($mother ne "") {
        $mgfather_id = $fam_husb{$indiv_famc{$mother_id}};
        $mgfather = $indiv_name{$mgfather_id};
        $mgmother_id = $fam_wife{$indiv_famc{$mother_id}};
        $mgmother = $indiv_name{$mgmother_id};
    }
}

#############################################
# Put in subroutine by Dale dePriest
# Sets $spouse_id, $spouse.
# Uses $indiv_id..
# Parameter family that spouse is in.
sub get_spouse_data {
    $this_fam = $_[0];
    # get spouse (assume opposite sex)
    if ($indiv_sex{$indiv_id} eq "M") {
        $spouse_id = $fam_wife{$this_fam};
    }
    elsif ($indiv_sex{$indiv_id} eq "F") {
        $spouse_id = $fam_husb{$this_fam};
    }
    $spouse = $indiv_name{$spouse_id};
}

#############################################
# Put in subroutine by Dale dePriest
# Sets @child.
# Uses %fam_chil.
# Parameter family that children are in.
sub get_child_data {
    $this_fam = $_[0];
    # get children (in order of fam entry)
    if ($this_fam) {
        # Split the children list into @child
        @child = split(/@/, $fam_chil{$this_fam});
        $num_children = @child;
    }
}

#############################################
# Make birth information of an individual private
# Parameter individual ID number
sub make_birt_private {
    $indiv_birt_date{$_[0]} = "$str_private";
    $indiv_birt_plac{$_[0]} = "";
}

#############################################
# Make a string lower case
sub lc {
    $s = pop(@_);
    tr/A-Z/a-z/;
}

sub get_note_link {
    my $parse_string = shift;
    if ($parse_string =~ /(http:.*)<br>/) {
        return "<a href=\"$1\" target=\"_top\">$`</a><br>";
    } else {
        return $parse_string;
    }
}

sub add_switch {
    my $id = pop(@_);

    $switch_str .= 
      "case \"$id\":\n";

    # birth date
    if ($indiv_birt_date{$id}) {
      $switch_str .= 
        "top.fframe.document.vform.birthd.value = \"$indiv_birt_date{$id}\"\n";
    }

    # birth place
    if ($indiv_birt_plac{$id}) {
      $switch_str .= 
        "top.fframe.document.vform.birthp.value = \"$indiv_birt_plac{$id}\"\n";
    }
    
    # death date
    if ($indiv_deat_date{$id}) {
      $switch_str .= 
        "top.fframe.document.vform.deathd.value = \"$indiv_deat_date{$id}\"\n";
    }

    # death place
    if ($indiv_deat_plac{$id}) {
      $switch_str .= 
        "top.fframe.document.vform.deathp.value = \"$indiv_deat_plac{$id}\"\n";
    }
    
    # burial date
    if ($indiv_buri_date{$id}) {
      $switch_str .= 
        "top.fframe.document.vform.buriald.value = \"$indiv_buri_date{$id}\"\n";
    }

    # burial place
    if ($indiv_buri_plac{$id}) {
      $switch_str .= 
        "top.fframe.document.vform.burialp.value = \"$indiv_buri_plac{$id}\"\n";
    }
    
    if ($lds_ord) {

        # baptism date
        if ($indiv_bap_date{$id}) {
            $switch_str .= 
              "top.fframe.document.vform.bapd.value = \"$indiv_bap_date{$id}\"\n";
        }

        # baptism place
        if ($indiv_bap_plac{$id}) {
            $switch_str .= 
              "top.fframe.document.vform.bapp.value = \"$indiv_bap_plac{$id}\"\n";
        }

        # endowment date
        if ($indiv_end_date{$id}) {
            $switch_str .= 
              "top.fframe.document.vform.endd.value = \"$indiv_end_date{$id}\"\n";
        }

        # endowment place
        if ($indiv_end_plac{$id}) {
            $switch_str .= 
              "top.fframe.document.vform.endp.value = \"$indiv_end_plac{$id}\"\n";
        }

        # sealing of child to parents date
        if ($indiv_slgc_date{$id}) {
            $switch_str .= 
            "top.fframe.document.vform.slgcd.value = \"$indiv_slgc_date{$id}\"\n";
        }

        # sealing of child to parents place
        if ($indiv_slgc_plac{$id}) {
            $switch_str .= 
            "top.fframe.document.vform.slgcp.value = \"$indiv_slgc_plac{$id}\"\n";
        }
    }

    $switch_str .= "break\n";
}

sub add_div {
    my $id = pop(@_);
    $div_str = $div_str . "<DIV ID=\"$id\" STYLE=\"position:absolute; ";
    $div_str = $div_str . "right:5; top:5; visibility:hidden;\">\n";
    $div_str = $div_str . "<table border=1 cellspacing=0 cellpadding=5>\n";
    $div_str = $div_str . "<tr><td align=left bgcolor=#ffff7f>\n";

    $div_str = $div_str . &create_vital_info($id);

    $div_str = $div_str . "</td></tr></table>";
    $div_str = $div_str . "</DIV>\n";
}

sub create_vital_info {
    my $id = pop(@_);
    my $vital_str = "";

    # birth date and place
    if ($indiv_birt{$id}) {
        $vital_str = "<b>$str_birth </b>";

        if ($indiv_birt_date{$id}) {
            $vital_str = $vital_str . "$indiv_birt_date{$id}";
            if ($indiv_birt_plac{$id})
                {$vital_str = $vital_str . ", ";}
        }
        if ($indiv_birt_plac{$id}) {
            $vital_str = $vital_str . "$indiv_birt_plac{$id}";
        }
        $vital_str = $vital_str . "<br>\n";
    }
    
    # death date and place
    if ($indiv_deat{$id}) {
        $vital_str = $vital_str . "<b>$str_death </b>";
        if ($indiv_deat_date{$id}) {
            $vital_str = $vital_str . "$indiv_deat_date{$id}";
            if ($indiv_deat_plac{$id})
                {$vital_str = $vital_str . ", ";}
        }
        if ($indiv_deat_plac{$id}) {
            $vital_str = $vital_str . "$indiv_deat_plac{$id}";
        }
        $vital_str = $vital_str . "<br>\n";
    }

    # burial date and place
    if ($indiv_buri{$id}) {
        $vital_str = $vital_str . "<b>$str_burial </b>";
        if ($indiv_buri_date{$id}) {
            $vital_str = $vital_str . "$indiv_buri_date{$id}";
            if ($indiv_buri_plac{$id})
                {$vital_str = $vital_str . ", ";}
        }
        if ($indiv_buri_plac{$id}) {
            $vital_str = $vital_str . "$indiv_buri_plac{$id}";
        }
        $vital_str = $vital_str . "<br>\n";
    }
    
    # LDS Ordinances
    if ($lds_ord) {
        if ($indiv_bap{$id}) {
            $vital_str = $vital_str . "<b>$str_bap </b>";
            if ($indiv_bap_date{$id}) {
                $vital_str = $vital_str . "$indiv_bap_date{$id}";
                if ($indiv_bap_plac{$id}) {
                    $vital_str = $vital_str . ", ";
                }
            }
            if ($indiv_bap_plac{$id}) {
                $vital_str = $vital_str . "$indiv_bap_plac{$id}";
            }
            $vital_str = $vital_str . "<br>\n";
        }
        if ($indiv_end{$id}) {
            $vital_str = $vital_str . "<b>$str_end </b>";
            if ($indiv_end_date{$id}) {
                $vital_str = $vital_str . "$indiv_end_date{$id}";
                if ($indiv_end_plac{$id}) {
                    $vital_str = $vital_str . ", ";
                }
            }
            if ($indiv_end_plac{$id}) {
                $vital_str = $vital_str . "$indiv_end_plac{$id}";
            }
            $vital_str = $vital_str . "<br>\n";
        }
        if ($indiv_slgc{$id}) {
            $vital_str = $vital_str . "<b>$str_slgc </b>";
            if ($indiv_slgc_date{$id}) {
                $vital_str = $vital_str . "$indiv_slgc_date{$id}";
                if ($indiv_slgc_plac{$id}) {
                    $vital_str = $vital_str . ", ";
                }
            }
            if ($indiv_slgc_plac{$id}) {
                $vital_str = $vital_str . "$indiv_slgc_plac{$id}";
            }
            $vital_str = $vital_str . "<br>\n";
        }
    }

    # occupation
    if ($indiv_occu{$id}) {
        $vital_str = $vital_str . "<b>$str_occupation</b> $indiv_occu{$id}\n";
    }

    return ($vital_str);
}

sub ped_print {
    my $node          = @_[0];
    my $level         = @_[1];
    my $add_td        = @_[2];
    my $end_add_td    = @_[3];
    my $print_tr      = @_[4];

    my $f_rowspan = $ped_father_level[$node] +1;
    my $m_rowspan = $ped_mother_level[$node] +1;

    if ($debug) {
        print DBUG_OUT "I:" . $node . " " . $level . " " . "\n"; 
    }

    # Follow father path

    if ($ped_father_level[$node] > 0) {
        if (($level > 1) && ($node % 2)) {
            # If odd going up, need to print image 

            if ($print_tr) {$ped_out_str .= "<TR>";}
            
            for ($col = 1; $col < $add_td; $col++) {
                $ped_out_str .= "<TD></TD>";
            }
            
            $ped_out_str .= "<TD valign=bottom align=right ";
            $ped_out_str .= "rowspan=$f_rowspan>";
            $ped_out_str .= "<img src=$gif_path/b$f_rowspan.gif></TD>";
            
            ped_print ($node*2, $level+1, $add_td, 
                         $end_add_td-$add_td+1, 0);
            $print_tr = 1;
        } else {
            # Even going up
            ped_print ($node*2, $level+1, $add_td+1, 
                         $end_add_td+1, $print_tr);
            $print_tr = 1;
        }
    }

    # Print self
    if ($print_tr) {$ped_out_str .= "<TR>";}

    if (($ped_father_level[$node] > 0) || ($node % 2)) {
        for ($col = 1; $col < $add_td; $col++) {
            $ped_out_str .= "<TD></TD>";
        }
    } else {
        for ($col = 1; $col < $end_add_td; $col++) {
            $ped_out_str .= "<TD></TD>";
        }
    }

    # If even, print image now
    if (($level > 1) && !($node % 2)) {
        $ped_out_str .= "<TD valign=top align=right rowspan=$m_rowspan>";
        $ped_out_str .= "<img src=$gif_path/t$m_rowspan.gif></TD>";
    }   

    # If odd and at final level, print image now
    if (($level > 1)  && ($node % 2)&& 
        ($ped_father_level[$node] == 0) && 
        ($ped_mother_level[$node] == 0)) {
        $ped_out_str .= "<TD valign=bottom align=right>";
        $ped_out_str .= "<img src=$gif_path/b1.gif></TD>";
    }   

    $ped_out_str .= "<TD colspan=8 valign=bottom>";
    $ped_out_str .= "<a href=\"p$ped_indiv_id[$node].html\"\n";
    $ped_out_str .= "onMouseOver = \"show('$ped_indiv_id[$node]')\"\n";
    $ped_out_str .= "onMouseOut = \"hide('$ped_indiv_id[$node]')\">\n";
    $ped_out_str .= "$indiv_name{$ped_indiv_id[$node]}</a>\n"; 

    # Indicate End of Line
    if ($ped_eol[$node]) {
      $ped_out_str .= "<img src=$gif_path/eol.gif>\n";
    }

    $ped_out_str .= "</TD>\n";

    add_div($ped_indiv_id[$node]);
    add_switch($ped_indiv_id[$node]);

    if ($ped_mother_level[$node] > 0) {
        if (($node % 2)) {
            # If odd going down, bump $add_td
            ped_print ($node*2+1, $level+1, $add_td+1, 
                         $end_add_td+1, 1);
        } else {
            # Even going down
            ped_print ($node*2+1, $level+1, $add_td, 
                         $end_add_td, 1);
        }
    }
}

sub ped_parse {
    my $my_indiv_id   = @_[0];
    my $node          = @_[1];
    my $level         = @_[2];

    my $ped_famc = $indiv_famc{$my_indiv_id};
    my $ped_father_id = $fam_husb{$ped_famc};
    my $ped_mother_id = $fam_wife{$ped_famc};

    my $father_return_level = 0;
    my $mother_return_level = 0;
    my $father_return_depth = 0;
    my $mother_return_depth = 0;
    my $my_depth;

    if ($debug) {
        print DBUG_OUT "I:" . $my_indiv_id . " F:" . $ped_father_id . " M:" . $ped_mother_id . " " . $node . " " . $level . " " . "\n"; 
    }

    # Follow father path

    if (($ped_father_id ne "") && ($level < $ped_generations)) {
      ($father_return_level, $father_return_depth) = 
        ped_parse ($ped_father_id, $node*2, $level+1);
    }

    # Follow mother path

    if (($ped_mother_id ne "") && ($level < $ped_generations)) {
      ($mother_return_level, $mother_return_depth) = 
        ped_parse ($ped_mother_id, $node*2+1, $level+1);
    }

    $ped_indiv_id[$node] = $my_indiv_id;
    $ped_father_level[$node] = $father_return_level;
    $ped_mother_level[$node] = $mother_return_level;

    # End of the line
    $ped_eol[$node] = ($ped_father_id eq "") && ($ped_mother_id eq "");

    if ($father_return_depth > $mother_return_depth) {
      $my_depth = $father_return_depth +1;
    } else {
      $my_depth = $mother_return_depth +1;
    } 
      
    return ($father_return_level + $mother_return_level +1, 
           $my_depth);
}

sub button_bar {
    my $my_bar_type   = @_[0];
    my $my_id         = @_[1];
    my $my_row        = @_[2];

    my $bstr .= "";

    if ($my_bar_type eq 'ped') {
        $bstr .= "<a href=\"$my_id.html\"\n";
        $bstr .= "onMouseDown = \"document.images['imGrp$my_row'].src=";
        $bstr .= "'$gif_path/group_dn.gif'\"\n";
        $bstr .= "onMouseOut = \"document.images['imGrp$my_row'].src=";
        $bstr .= "'$gif_path/group_up.gif'\"\n";
        $bstr .= "onMouseOver = \"document.images['imGrp$my_row'].src=";
        $bstr .= "'$gif_path/group_act.gif'\">\n";
        $bstr .= "<img name=\"imGrp$my_row\" src=$gif_path/group_up.gif alt=";
        $bstr .= "\"Family Group\" border=0></a>\n";

        $bstr .= "<a>\n";
        $bstr .= "<img name=\"imPed$my_row\" src=$gif_path/pedigree_dn.gif alt=";
        $bstr .= "\"Pedigree Chart\" border=0></a>\n";

        $bstr .= "<a href=\"people.html\"\n";
        $bstr .= "onMouseDown = \"document.images['imPpl$my_row'].src=";
        $bstr .= "'$gif_path/people_dn.gif'\"\n";
        $bstr .= "onMouseOut = \"document.images['imPpl$my_row'].src=";
        $bstr .= "'$gif_path/people_up.gif'\"\n";
        $bstr .= "onMouseOver = \"document.images['imPpl$my_row'].src=";
        $bstr .= "'$gif_path/people_act.gif'\">\n";
        $bstr .= "<img name=\"imPpl$my_row\" src=$gif_path/people_up.gif alt=";
        $bstr .= "\"People\" border=0></a>\n";
        
        $bstr .= "<a href=\"surnames.html\"\n";
        $bstr .= "onMouseDown = \"document.images['imSur$my_row'].src=";
        $bstr .= "'$gif_path/surname_dn.gif'\"\n";
        $bstr .= "onMouseOut = \"document.images['imSur$my_row'].src=";
        $bstr .= "'$gif_path/surname_up.gif'\"\n";
        $bstr .= "onMouseOver = \"document.images['imSur$my_row'].src=";
        $bstr .= "'$gif_path/surname_act.gif'\">\n";
        $bstr .= "<img name=\"imSur$my_row\" src=$gif_path/surname_up.gif alt=";
        $bstr .= "\"Surname\" border=0></a>\n";
    } 
    elsif ($my_bar_type eq 'grp') {
        $bstr .= "<a>\n";
        $bstr .= "<img name=\"imGrp$my_row\" src=$gif_path/group_dn.gif alt=";
        $bstr .= "\"Family Group\" border=0></a>\n";

        $bstr .= "<a href=\"p$my_id.html\"\n";
        $bstr .= "onMouseDown = \"document.images['imPed$my_row'].src=";
        $bstr .= "'$gif_path/pedigree_dn.gif'\"\n";
        $bstr .= "onMouseOut = \"document.images['imPed$my_row'].src=";
        $bstr .= "'$gif_path/pedigree_up.gif'\"\n";
        $bstr .= "onMouseOver = \"document.images['imPed$my_row'].src=";
        $bstr .= "'$gif_path/pedigree_act.gif'\">\n";
        $bstr .= "<img name=\"imPed$my_row\" src=$gif_path/pedigree_up.gif alt=";
        $bstr .= "\"Pedigree Chart\" border=0></a>\n";

        $bstr .= "<a href=\"people.html\"\n";
        $bstr .= "onMouseDown = \"document.images['imPpl$my_row'].src=";
        $bstr .= "'$gif_path/people_dn.gif'\"\n";
        $bstr .= "onMouseOut = \"document.images['imPpl$my_row'].src=";
        $bstr .= "'$gif_path/people_up.gif'\"\n";
        $bstr .= "onMouseOver = \"document.images['imPpl$my_row'].src=";
        $bstr .= "'$gif_path/people_act.gif'\">\n";
        $bstr .= "<img name=\"imPpl$my_row\" src=$gif_path/people_up.gif alt=";
        $bstr .= "\"People\" border=0></a>\n";
        
        $bstr .= "<a href=\"surnames.html\"\n";
        $bstr .= "onMouseDown = \"document.images['imSur$my_row'].src=";
        $bstr .= "'$gif_path/surname_dn.gif'\"\n";
        $bstr .= "onMouseOut = \"document.images['imSur$my_row'].src=";
        $bstr .= "'$gif_path/surname_up.gif'\"\n";
        $bstr .= "onMouseOver = \"document.images['imSur$my_row'].src=";
        $bstr .= "'$gif_path/surname_act.gif'\">\n";
        $bstr .= "<img name=\"imSur$my_row\" src=$gif_path/surname_up.gif alt=";
        $bstr .= "\"Surname\" border=0></a>\n";
    }
    elsif ($my_bar_type eq 'sur') {
        $bstr .= "<a>\n";
        $bstr .= "<img name=\"imGrp$my_row\" src=$gif_path/group_gr.gif alt=";
        $bstr .= "\"Family Group\" border=0></a>\n";

        $bstr .= "<a>\n";
        $bstr .= "<img name=\"imPed$my_row\" src=$gif_path/pedigree_gr.gif alt=";
        $bstr .= "\"Pedigree Chart\" border=0></a>\n";

        $bstr .= "<a href=\"people.html\"\n";
        $bstr .= "onMouseDown = \"document.images['imPpl$my_row'].src=";
        $bstr .= "'$gif_path/people_dn.gif'\"\n";
        $bstr .= "onMouseOut = \"document.images['imPpl$my_row'].src=";
        $bstr .= "'$gif_path/people_up.gif'\"\n";
        $bstr .= "onMouseOver = \"document.images['imPpl$my_row'].src=";
        $bstr .= "'$gif_path/people_act.gif'\">\n";
        $bstr .= "<img name=\"imPpl$my_row\" src=$gif_path/people_up.gif alt=";
        $bstr .= "\"People\" border=0></a>\n";
        
        $bstr .= "<a>\n";
        $bstr .= "<img name=\"imSur$my_row\" src=$gif_path/surname_dn.gif alt=";
        $bstr .= "\"Surname\" border=0></a>\n";
    }
    elsif ($my_bar_type eq 'ppl') {
        $bstr .= "<a>\n";
        $bstr .= "<img name=\"imGrp$my_row\" src=$gif_path/group_gr.gif alt=";
        $bstr .= "\"Family Group\" border=0></a>\n";

        $bstr .= "<a>\n";
        $bstr .= "<img name=\"imPed$my_row\" src=$gif_path/pedigree_gr.gif alt=";
        $bstr .= "\"Pedigree Chart\" border=0></a>\n";

        $bstr .= "<a>\n";
        $bstr .= "<img name=\"imPpl$my_row\" src=$gif_path/people_dn.gif alt=";
        $bstr .= "\"People\" border=0></a>\n";
        
        $bstr .= "<a href=\"surnames.html\"\n";
        $bstr .= "onMouseDown = \"document.images['imSur$my_row'].src=";
        $bstr .= "'$gif_path/surname_dn.gif'\"\n";
        $bstr .= "onMouseOut = \"document.images['imSur$my_row'].src=";
        $bstr .= "'$gif_path/surname_up.gif'\"\n";
        $bstr .= "onMouseOver = \"document.images['imSur$my_row'].src=";
        $bstr .= "'$gif_path/surname_act.gif'\">\n";
        $bstr .= "<img name=\"imSur$my_row\" src=$gif_path/surname_up.gif alt=";
        $bstr .= "\"Surname\" border=0></a>\n";
    }

    return $bstr;
}
